package com.webchat.ugc.service.moment;

import com.webchat.common.enums.AccountRelationTypeEnum;
import com.webchat.common.enums.RedisKeyEnum;
import com.webchat.common.service.RedisService;
import com.webchat.ugc.repository.dao.IMomentTimeLineDAO;
import com.webchat.ugc.repository.entity.MomentTimeLineEntity;
import com.webchat.ugc.service.AccountService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;


@Service
public class MomentTimeLineService {


    @Autowired
    private IMomentTimeLineDAO momentTimeLineDAO;
    @Autowired
    private RedisService redisService;
    @Autowired
    private AccountService accountService;


    /**
     * TODO  数据一致性保证
     *
     * @param author
     * @param momentId
     * @param time
     */
    public void addTimeLine(String author, Long momentId, Long time, boolean writeFriends) {
        Set<String> friends = new HashSet<>();
        friends.add(author);
        if (writeFriends) {
            friends = accountService.getAllSubscriberByAccount(author, AccountRelationTypeEnum.USER_USER);
        }
        if (CollectionUtils.isEmpty(friends)) {
            return;
        }

        // TODO BY策略过滤不需要写入的用户

        Date publishDate = new Date(time);
        // 持计划话朋友圈数据
        List<MomentTimeLineEntity> momentTimeLineEntities = friends.stream().map(u -> {
            MomentTimeLineEntity momentTimeLineEntity = new MomentTimeLineEntity();
            momentTimeLineEntity.setUserId(u);
            momentTimeLineEntity.setMomentId(momentId);
            momentTimeLineEntity.setTimeLine(publishDate);
            return momentTimeLineEntity;
        }).collect(Collectors.toList());
        this.saveMomentTimeLine(momentTimeLineEntities);

        // 写时间线缓存
        this.saveMomentTimeLineCache(friends, momentId, time);
    }

    private void saveMomentTimeLineCache(Set<String> friends, Long momentId, Long time) {

        List<String> keys = friends.stream().map( u ->
                RedisKeyEnum.MOMENT_TIME_LINE_CACHE.getKey(u)).collect(Collectors.toList());
        keys.forEach(key ->
            redisService.zadd(key, String.valueOf(momentId), time,
                         RedisKeyEnum.MOMENT_TIME_LINE_CACHE.getExpireTime()));
    }

    private void saveMomentTimeLine(List<MomentTimeLineEntity> momentTimeLineEntities) {
        if (CollectionUtils.isEmpty(momentTimeLineEntities)) {
            return;
        }
        momentTimeLineDAO.saveAll(momentTimeLineEntities);
    }


    public void save(String userId) {
        MomentTimeLineEntity entity = new MomentTimeLineEntity();

        entity.setUserId(userId);
        entity.setMomentId(1L);
        entity.setTimeLine(new Date());
        momentTimeLineDAO.save(entity);
    }


//    public static void main(String[] args) {
//        String userId = "U_1bb8a10f2d2a45a4b075c20016f10cb8";
//        int[] idxArr = new int[3];
//        for (int i = 0; i < 1000; i++) {
//
//            userId += i;
//            int idx = Math.abs(userId.hashCode()) % 3;
//            idxArr[idx] = idxArr[idx] + 1;
//        }
//        System.out.println("0:" + idxArr[0]);
//        System.out.println("1:" + idxArr[1]);
//        System.out.println("2:" + idxArr[2]);
//    }

}
